/* 
 * $Id: io.c,v 1.3 2000/11/01 17:40:55 markus Exp $ */

/* Line thinning program */
/*   Input/output and file support functions */

/* Mike Baba */
/* DBA Systems */
/* Fairfax, Va */
/* Jan 1990 */

/* Jean Ezell */
/* US Army Corps of Engineers */
/* Construction Engineering Research Laboratory */
/* Modelling and Simulation Team */
/* Champaign, IL  61820 */
/* January - February 1988 */

/* Entry points: */
/*   get_a_row     get row from temporary (compressed) work space */
/*   open_file     open input cell file and read it into (compressed) work space */
/*   close_file    copy work file into new cell file */
/*   map_size      get size of map and its pad */

/* Global variables: */
/*   row_io        place to store pointer to row manager stuff */
/*   n_rows        number of rows in the work file (includes pads) */
/*   n_cols        number of columns in the work file (includes pads) */

#include <string.h>
#include <unistd.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <zlib.h>
#include "gis.h"
#include "local_proto.h"

#define PAD 2
#define MAX_ROW 7

/*extern int errno; */ /* included #include <errno.h> instead 1/2000*/
extern char *error_prefix;
static int n_rows, n_cols;
static struct zrow *work;
static CELL *buffer;

int get_a_row (int row, CELL *buf) {
  uLong len = n_cols * sizeof(CELL);
  if (row < 0 || row >= n_rows)
    return 0;
  if(work[row].data == NULL) {
    fprintf(stderr, "Error: no data stored row: %d\n", row);
    exit(0);
  }
  uncompress((Bytef *)buf, &len, work[row].data, (uLong)work[row].size);
  return 1;
}

int put_a_row( int row, CELL *buf)
{
  int i;
  uLong len = n_cols * sizeof(CELL) * 2;

  compress((Bytef *)buffer, &len,	   
	   (Bytef *)buf, n_cols * sizeof(CELL));
  G_free(work[row].data);
  work[row].data = (Bytef *)G_malloc(len * sizeof(char));
  memcpy((void *)(work[row].data), (void *)buffer, len);
  work[row].size = (int)len;
  return 1;
}

void swap(CELL *a, CELL *b) {
  CELL *t;
  t = a;
  a = b;
  b = t;
}

int open_file (char *name) {
  char *mapset;
  int cell_file, buf_len;
  int i, row, col;
  char cell[100];
  CELL *buf;
  buffer = (CELL *) G_malloc(n_cols * 2 * sizeof(CELL));
  /* open cell file */
  strcpy (cell, name);
  if ((mapset = G_find_cell2(cell,"")) == NULL)
    {
      fprintf(stderr,"%s:  open_file:  cell file %s not found\n",
	      error_prefix, name);
      exit(-1);
    }
  if ((cell_file = G_open_cell_old(cell,mapset)) < 0)
    {
      fprintf(stderr,"%s:  open_file:  could not open cell file %s in %s\n",
	      error_prefix, cell, mapset);
      exit(-1);
    }
  n_rows = G_window_rows();
  n_cols = G_window_cols();
  fprintf(stdout,"File %s -- %d rows X %d columns\n",name,n_rows,n_cols);
  n_cols += (PAD << 1);
  /* copy cell file into our read/write file */
  
  buf = (CELL *) G_malloc(buf_len = n_cols * sizeof(CELL));
  work = (struct zrow *) G_malloc((n_rows + 2 * PAD) * 
				  sizeof(struct zrow));
  for (row = 0; row < n_rows + 2 * PAD; row++) {
    work[row].data = NULL;
    work[row].size = 0;
  }
  for (col = 0; col < n_cols; col++)
    buf[col] = 0;
  for (i = 0; i < PAD; i++)
    put_a_row(i, buf);
  for (row = 0; row < n_rows; row++)
    {
      if (G_get_map_row(cell_file, buf + PAD,row) < 0)
	{
	  fprintf(stderr,"%s:  open_file:  error reading from %s in %s\n",
		  error_prefix, cell, mapset);
	  exit(-1);
	}
      put_a_row(row + PAD, buf);
    }
  for (col = 0; col < n_cols; col++)
    buf[col] = 0;
  for (i = 0; i < PAD; i++)
    put_a_row(n_rows + PAD + i, buf);
  n_rows += (PAD << 1);
  G_free(buf);
  G_close_cell(cell_file);
  return 0;
}

int 
close_file (char *name)
{
  int cell_file, row, k;
  int row_count, col_count;
  CELL *buf;
  buf = (CELL *) G_malloc(n_cols * sizeof(CELL));
  if ((cell_file = G_open_cell_new(name)) < 0)
    {
      fprintf(stderr,"%s:  close_cell:  could not open output file %s\n",
	      error_prefix,name);
      exit(-1);
    }
  row_count = n_rows - (PAD << 1);
  col_count = n_cols - (PAD << 1);
  fprintf(stdout,"Output file %d rows X %d columns\n",
	  row_count, col_count);
  fprintf(stdout,"Window %d rows X %d columns\n",G_window_rows(),G_window_cols());
  for (row = 0, k = PAD; row < row_count; row++, k++)
    {
      get_a_row(k, buf);
      G_put_map_row(cell_file,buf + PAD);
    }
  G_close_cell(cell_file);
  G_free(buf);
  return 0;
}

int map_size (int *r, int *c, int *p) {
  *r = n_rows;
  *c = n_cols;
  *p = PAD;
  
  return 0;
}
